#version 330

in vec3 eye_direction;
in vec2 UV;
in vec2 vTexcoord;
in vec3 vViewRay;

//layout(location = 0) out vec4 color;
layout(location = 0) out vec4 result;

uniform sampler2D blurTexture;
uniform sampler2D uTexLinearDepth;

uniform float nearPlane;
uniform float farPlane;

uniform mat4 uInverseModelViewMat; // inverse model->view
uniform mat4 uPrevModelViewProj; // previous model->view->projection
uniform int nSamples;
uniform float mblurScale;



void main(){

	 // get current world space position:
      vec3 current = vViewRay * texture2D(uTexLinearDepth, vTexcoord).r;
      current = vec3(uInverseModelViewMat * vec4(current, 1.0));
 
      // get previous screen space position:
      vec4 previous = uPrevModelViewProj * vec4(current, 1.0);
      previous.xyz /= previous.w;
      previous.xy = previous.xy * 0.5 + 0.5;

      vec2 blurVec = previous.xy - vTexcoord;
	  
	  blurVec *= mblurScale; 

	  
      // perform blur:
    result = texture2D(blurTexture, vTexcoord);
       for (int i = 1; i < nSamples; ++i) {
       // get offset in range [-0.5, 0.5]:
      vec2 offset = blurVec * (float(i) / float(nSamples - 1) - 0.5);
  
     // sample & add to result:
      result += texture2D(blurTexture, vTexcoord + offset);
    }
 
    result /= float(nSamples);


	//vec3 depth=texture2D(texture, UV).xyz;
	//depth *= vec3(0.5,0.3,0.5);
	//color=vec4(depth, 1);
}