#version 330

uniform sampler2D colorTexture;
uniform float lightPosition[3];
uniform float lightColor[3];
uniform float lightPower;
uniform float lightExponent;
uniform float lightInnerAngle;
uniform float lightOuterAngle;

uniform vec3 specularColor;
uniform float shininess;

uniform sampler2D shadowMap;

uniform float nearPlane;
uniform float farPlane;

uniform float alpha;

in vec2 fUVs;
in vec3 Normal_cameraSpace;
in vec3 EyeDirection_cameraSpace;
in vec3 LightDirection_cameraSpace;
in vec3 Position_worldSpace;
in vec3 Position_cameraSpace;
in vec4 ShadowCoord;
in vec3 LightTarget_cameraSpace;

layout(location = 0) out vec4 fragColor;

void main()
{
	//Normalize vectors
	vec3 n=normalize(Normal_cameraSpace);
	
	vec3 textureColor=texture(colorTexture, fUVs).rgb;
	vec3 color=vec3(0.0, 0.0, 0.0);
	vec3 diffuseColor=vec3(0.0, 0.0, 0.0);
	vec3 specularColor=vec3(0.0, 0.0, 0.0);

	vec3 l=normalize(LightDirection_cameraSpace);

	vec3 lightPosVec=vec3(lightPosition[0], lightPosition[1], lightPosition[2]);
	float distance=length(lightPosVec-Position_worldSpace);
	
	//Diffuse color
	float cosTheta=clamp(dot(n, l), 0, 1);
	
	if(cosTheta>0)
	{
		float cosDir=dot(normalize(LightTarget_cameraSpace), -l);
		float spotEffect=smoothstep(lightOuterAngle, lightInnerAngle, cosDir);

			spotEffect=pow(spotEffect, lightExponent);
			
			diffuseColor =	textureColor
							* vec3(lightColor[0], lightColor[1], lightColor[2])
							* lightPower
							* cosTheta
							* spotEffect
							/ max((distance*distance), 50);

			//Eye vector
			vec3 e=normalize(EyeDirection_cameraSpace);
			//Reflection vector
			vec3 r=reflect(-l, n);

			//Specular color
			float cosAlpha=clamp(dot(e, r), 0, 1);
			specularColor =	specularColor
							* vec3(lightColor[0], lightColor[1], lightColor[2])
							* lightPower 
							* pow(cosAlpha, shininess)
							* spotEffect
							/ max((distance*distance), 25);
		
	}
	
	vec3 tex_coords = ShadowCoord.xyz/ShadowCoord.w;
	
	float depth=texture(shadowMap, tex_coords.xy).r;

	float visibility=0.0f;

	depth=(2*nearPlane)/(farPlane+nearPlane-depth*(farPlane-nearPlane));
	tex_coords.z=(2*nearPlane)/(farPlane+nearPlane-tex_coords.z*(farPlane-nearPlane));
	tex_coords.z-=0.005;

	if(depth < tex_coords.z)
		visibility=0.0f;
	else
		visibility=1.0f;

	//Ambient color
	vec3 ambientColor=vec3(0.2, 0.2, 0.2)*textureColor;

	color=	ambientColor+
			visibility*diffuseColor+
			visibility*specularColor;

	//Crosshair
	if(Position_cameraSpace.x>0.45 && Position_cameraSpace.x<0.55 &&
		Position_cameraSpace.y>0.45 && Position_cameraSpace.y<0.55)
			color+=vec3(1,-0.5,-0.5);

	fragColor = vec4(color, alpha);
}